/***************************************************************************
 *
 * knetworkmanager-encryption.cpp - A NetworkManager frontend for KDE 
 *
 * Copyright (C) 2006 Novell, Inc.
 *
 * Author: Timo Hoenig     <thoenig@suse.de>, <thoenig@nouse.net>
 *         Will Stephenson <wstephenson@suse.de>, <wstephenson@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **************************************************************************/

#include <kconfigbase.h>
#include <kdebug.h>
//
#include "knetworkmanager.h"
#include "knetworkmanager-encryption.h"
#include <NetworkManager/dbus-helpers.h>

void
Encryption::addCipher (IEEE_802_11_Cipher* cipher)
{
	_cipherList->append (cipher);
}

bool
Encryption::isValid (const QString & essid)
{
	QString              input = this->_secret["password"];
	bool                 status = false;

	if ((essid.length () == 0) || (input.length () == 0) || (!_cipherList) || (_cipherList->empty ())) {
		kdDebug() << " Encryption::isValid failed, bad inputs" << endl;
		kdDebug() << "isValid, essid: " << essid << ", input: " << input << " cipherlist: " << _cipherList << endl;
		return status;
	}

	CipherList::iterator iter;
	for (iter = _cipherList->begin (); iter != _cipherList->end (); ++iter) {
		if (ieee_802_11_cipher_validate (*iter, essid.ascii(), input.ascii() ) == 0) {
			this->_currentCipher = *iter;
			status = true;
		}
	}

	return status;
}
		
SecretMap
Encryption::getSecrets () const
{
	return _secret;
}

void
Encryption::setSecrets (const SecretMap & secret)
{
	_secret = secret;
}

Encryption::Encryption () : _currentCipher(0), _cipherList(0)
{
}

Encryption::~Encryption ()
{
	delete _cipherList;
}

/* EncryptionNone */

bool
EncryptionNone::isValid (const QString & essid)
{
	if (essid.length ())
		return true;
	else
		return false;
}

bool
EncryptionNone::serialize (DBusMessage* msg, const QString & essid)
{
	kdDebug() << k_funcinfo << endl;
	//printf ("SER: none\n");
	bool status = false;

	if (!msg || !essid)
		return false;

	status = nmu_security_serialize_none_with_cipher (msg);

	return status;
}

bool 
EncryptionNone::deserialize( DBusMessageIter *, int we_cipher )
{
	// do nothing 
	return ( we_cipher == IW_AUTH_CIPHER_NONE );
}

void
EncryptionNone::setDefaults (void)
{
	return;
}

EncryptionNone::EncryptionNone ()
{
	
}

EncryptionNone::~EncryptionNone ()
{

}

void EncryptionNone::persist( KConfigBase * cfg ) const
{
	cfg->writeEntry( "Encryption", "none" );
}

void EncryptionNone::restore( const KConfigBase * )
{
	kdDebug() << k_funcinfo << endl;
}

/* EncryptionWEP */

bool
EncryptionWEP::serialize (DBusMessage* msg, const QString & essid)
{
	kdDebug() << "serialize: msg: " << msg << " essid: " << essid << " isValid(): " << isValid( essid ) << " secret: " << _secret["password"] << " method: " << _method <<endl;
	//printf ("SER: wep\n");
	bool status = false;

	if (!msg || !essid || !isValid( essid ) ) {
		return false;
	}

	status = nmu_security_serialize_wep_with_cipher (msg, _currentCipher, essid.utf8(), _secret["password"].utf8(), _method);

	return status;
}

bool 
EncryptionWEP::deserialize( DBusMessageIter * iter, int we_cipher )
{
	char * key = 0;
	int	keyLen, authAlg;

	if ( iter == 0 )
		return false;
	if ( !( we_cipher == NM_AUTH_TYPE_WEP40 || we_cipher == NM_AUTH_TYPE_WEP104 ) )
		return false;

	if ( !nmu_security_deserialize_wep( iter, &key, &keyLen, &authAlg ) )
		return false;
	
	if ( !( authAlg == IW_AUTH_ALG_OPEN_SYSTEM || authAlg == IW_AUTH_ALG_SHARED_KEY ) )
		return false;
	
	setMethod( (WEPMethod) authAlg );
	
	// we don't store this key, as it is the hashed version
	return true;
}

// bool
// EncryptionWEP::serialize (DBusMessageIter* iter)
// {
// 	//printf ("SER: wep\n");
// 	bool status = false;
// 
// 	if (iter)
// 	{
// 		status = nmu_we_cipher_append_helper(iter, _currentCipher );
// 		status &= nmu_security_serialize_wep (iter, _secret, _method);
// 	}
// 
// 	return status;
// }

void
EncryptionWEP::setType( WEPType type )
{
	_type = type;
	delete _cipherList;
	_cipherList = new CipherList ();
	
	if (_type == WEP_ASCII) {
		addCipher (cipher_wep64_ascii_new  ());
		addCipher (cipher_wep128_ascii_new ());
	} else if (_type == WEP_HEX) {
		addCipher (cipher_wep64_hex_new  ());
		addCipher (cipher_wep128_hex_new ());
	} else if (_type == WEP_PASSPHRASE) {
		addCipher (cipher_wep64_passphrase_new  ());
		addCipher (cipher_wep128_passphrase_new ());
	}
}

void
EncryptionWEP::setDefaults ()
{
	//printf ("wep setting defaults\n");
	setMethod (WEP_OPEN_SYSTEM);
}

void
EncryptionWEP::setMethod (WEPMethod method)
{
	//printf ("setMethod %s\n", (method == WEP_SHARED_KEY)?"shared key":"open system");
	_method = method;
}

WEPMethod
EncryptionWEP::getMethod (void)
{
	return _method;
}


EncryptionWEP::EncryptionWEP (WEPType type)
{
	setType( type );

	setDefaults ();
}

EncryptionWEP::~EncryptionWEP ()
{

}

void EncryptionWEP::persist( KConfigBase * cfg ) const
{
	cfg->writeEntry( "Encryption", "WEP" );
	if ( WEP_ASCII == _type )
		cfg->writeEntry( "WEPType", "ASCII" );
	else if ( WEP_HEX == _type )
		cfg->writeEntry( "WEPType", "HEX" );
	else
		cfg->writeEntry( "WEPType", "PASSPHRASE" );
	
	if ( WEP_OPEN_SYSTEM == _method )
		cfg->writeEntry( "WEPMethod", "OpenSystem" );
	else
		cfg->writeEntry( "WEPMethod", "SharedKey" );
}

void EncryptionWEP::restore( const KConfigBase * cfg )
{
	kdDebug() << k_funcinfo << endl;
	QString wepType = cfg->readEntry( "WEPType", "ASCII" );
	if ( "ASCII" == wepType )
		setType( WEP_ASCII );
	else if ( "HEX" == wepType )
		setType( WEP_HEX );
	else
		setType( WEP_PASSPHRASE );
		
	QString wepMethod = cfg->readEntry( "WEPMethod", "OpenSystem" );

	if ( "OpenSystem" == wepMethod )
		_method = WEP_OPEN_SYSTEM;
	else
		_method = WEP_SHARED_KEY;
}

/* EncryptionWPAPersonal */

bool
EncryptionWPAPersonal::serialize (DBusMessage* msg, const QString & essid)
{
	kdDebug() << k_funcinfo << endl;
	// printf ("SER: WPA\n");
	bool status = false;

	if (!msg || !essid || !isValid( essid ) )
		return false;

	status = nmu_security_serialize_wpa_psk_with_cipher (msg, _currentCipher, essid.utf8(), _secret["password"].utf8(), _version, IW_AUTH_KEY_MGMT_PSK);

	return status;
}

bool 
EncryptionWPAPersonal::deserialize( DBusMessageIter * iter, int we_cipher )
{
	char * key;
	int	keyLen, wpaVersion, keyManagement;

	if ( !iter )
		return false;
	if ( !(we_cipher == NM_AUTH_TYPE_WPA_PSK_TKIP || we_cipher == NM_AUTH_TYPE_WPA_PSK_CCMP || we_cipher == NM_AUTH_TYPE_WPA_PSK_AUTO) )
		return false;

	if (!nmu_security_deserialize_wpa_psk (iter, &key, &keyLen, &wpaVersion, &keyManagement) )
		return false;
	
	if ( !(wpaVersion == IW_AUTH_WPA_VERSION_WPA || wpaVersion == IW_AUTH_WPA_VERSION_WPA2 ) )
		return false;
	
	if ( keyManagement != IW_AUTH_KEY_MGMT_PSK )
		return false;
	
	setVersion( (WPAVersion)wpaVersion );
	
	// we don't store this key, as it is the hashed version
	return true;
}

void
EncryptionWPAPersonal::setDefaults (void)
{
	/* Once NM can default to "no default protocol" change this */
	setProtocol (WPA_AUTO);
	setVersion  (WPA1);
}

void
EncryptionWPAPersonal::setProtocol (WPAProtocol protocol)
{
	_protocol = protocol;

	/* switch ciphers to match with protocol */
	cipher_wpa_psk_hex_set_we_cipher        ((IEEE_802_11_Cipher*) *_cipherList->at(0), _protocol);
	cipher_wpa_psk_passphrase_set_we_cipher ((IEEE_802_11_Cipher*) *_cipherList->at(1), _protocol);
}

WPAProtocol
EncryptionWPAPersonal::getProtocol (void)
{
	return _protocol;
}

void
EncryptionWPAPersonal::setVersion (WPAVersion version)
{
	_version = version;
}

WPAVersion
EncryptionWPAPersonal::getVersion (void)
{
	return _version;
}

EncryptionWPAPersonal::EncryptionWPAPersonal ()
{
	_cipherList = new CipherList (); 
	addCipher (cipher_wpa_psk_hex_new ());
	addCipher (cipher_wpa_psk_passphrase_new ());

	setDefaults ();
}

EncryptionWPAPersonal::~EncryptionWPAPersonal ()
{

}

void EncryptionWPAPersonal::persist( KConfigBase * cfg ) const
{
	cfg->writeEntry( "Encryption", "WPA" );
	if ( WPA_AUTO == _protocol )
		cfg->writeEntry( "WPAProtocol", "AUTO" );
	if ( WPA_TKIP == _protocol )
		cfg->writeEntry( "WPAProtocol", "TKIP" );
	else if (WPA_CCMP_AES == _protocol)
		cfg->writeEntry( "WPAProtocol", "CCMPAES" );
	
	if ( WPA1 == _version )
		cfg->writeEntry( "WPAVersion", "WPA1" );
	else
		cfg->writeEntry( "WPAVersion", "WPA2" );

}

void EncryptionWPAPersonal::restore( const KConfigBase * cfg )
{
	kdDebug() << k_funcinfo << endl;

	QString wpaProtocol = cfg->readEntry( "WPAProtocol", "TKIP" );
	if ( "AUTO" == wpaProtocol )
		_protocol = WPA_AUTO;
	else if ( "TKIP" == wpaProtocol )
		_protocol = WPA_TKIP;
	else
		_protocol = WPA_CCMP_AES;
		
	QString wpaVersion = cfg->readEntry( "WPAVersion", "WPA1" );

	if ( "WPA1" == wpaVersion )
		_version = WPA1;
	else
		_version = WPA2;
}

/* EncryptionWPAEnterprise */

QString EncryptionWPAEnterprise::IdPasswordKey = "identityPassword";
QString EncryptionWPAEnterprise::CertPrivatePasswordKey = "certPrivatePassword";

bool
EncryptionWPAEnterprise::serialize (DBusMessage* msg, const QString & essid)
{
	kdDebug() << k_funcinfo << endl;
	DBusMessageIter iter;
	bool status = false;

	/* FIXME for RC1 */
	int keyType = NM_AUTH_TYPE_WPA_PSK_AUTO;

	if (!msg || !essid || !isValid( essid ) )
		return false;

	/* FIXME: If the user denies access to kwallet we'd send NULL over the bus.  Avoid that. */
	if (!_secret[IdPasswordKey])
		_secret[IdPasswordKey] = "";

	if (!_secret[CertPrivatePasswordKey])
		_secret[CertPrivatePasswordKey] = "";

	kdDebug () << "method: " << _method << " identity: " << _identity << " password: " << _secret[IdPasswordKey] << " anon ident: " << _anonIdentity << " cert priv passwd: "  << _secret[CertPrivatePasswordKey] << " cert priv: " << _certPrivate << " cert client: " << _certClient << " cert CA: " << _certCA  <<  " version: " << _version << endl;

	dbus_message_iter_init_append (msg, &iter);
	status = nmu_security_serialize_wpa_eap_with_cipher (&iter, _method, keyType, _identity.utf8(), _secret[IdPasswordKey].utf8(), _anonIdentity.utf8(),
							     _secret[CertPrivatePasswordKey].utf8(), _certPrivate.utf8(), _certClient.utf8(), _certCA.utf8(), _version);

	return status;
}

bool 
EncryptionWPAEnterprise::deserialize( DBusMessageIter * iter, int we_cipher )
{
//	kdDebug() << k_funcinfo << endl;
	int method;
	/* FIXME for RC1 */
	int keyType;
	char* identity = NULL;
	char* identityPassword = NULL;
	char* anonIdentity = NULL;
	char* certPrivatePassword = NULL;
	char* certPrivate = NULL;
	char* certClient = NULL;
	char* certCA = NULL;
	int version;

	if ( !iter )
		return false;
	if ( !(we_cipher == NM_AUTH_TYPE_WPA_EAP ))
		return false;

	if (!nmu_security_deserialize_wpa_eap (iter, &method, &keyType, &identity, &identityPassword, &anonIdentity, &certPrivatePassword,
					       &certPrivate, &certClient, &certCA, &version) )
		return false;
	
	if ( !(version == IW_AUTH_WPA_VERSION_WPA || version == IW_AUTH_WPA_VERSION_WPA2 ) )
		return false;
	if ( !(method == EAP_PEAP || method == EAP_TLS || method == EAP_TTLS ) )
		return false;

	setMethod ((EAPMethod) method);
	setIdentity (identity);
	setAnonIdentity (anonIdentity);
	setCertPrivate (certPrivate);
	setCertClient (certClient);
	setCertCA (certCA);
	setVersion ((WPAVersion) version);
	SecretMap map;
	map.insert( IdPasswordKey, identityPassword );
	map.insert( CertPrivatePasswordKey, certPrivatePassword );
	setSecrets( map );
	
	return true;
}

bool
EncryptionWPAEnterprise::isValid (const QString & essid)
{
//	kdDebug() << k_funcinfo << endl;
	if (essid.length ())
		return true;
	else
		return false;
}

void
EncryptionWPAEnterprise::setDefaults (void)
{
//	kdDebug() << k_funcinfo << endl;
	/* Once NM can default to "no default protocol" change this */
	setProtocol (WPA_EAP);
	setVersion  (WPA1);
	setMethod   (EAP_PEAP);

	setIdentity ("");
	setAnonIdentity ("");
	setCertPrivate ("");
	setCertClient ("");
	setCertCA ("");
	SecretMap map;
	map.insert( IdPasswordKey, "" );
	map.insert( CertPrivatePasswordKey, "" );
	setSecrets( map );
}

void
EncryptionWPAEnterprise::setIdentity (const QString & identity)
{
	kdDebug() << k_funcinfo << endl;
	_identity = identity;
}

QString
EncryptionWPAEnterprise::getIdentify (void) const
{
	kdDebug() << k_funcinfo << endl;
	return _identity;
}

void
EncryptionWPAEnterprise::setAnonIdentity (const QString & anonIdentity)
{
	kdDebug() << k_funcinfo << endl;
	_anonIdentity = anonIdentity;
}

QString
EncryptionWPAEnterprise::getAnonIdentity (void) const
{
	kdDebug() << k_funcinfo << endl;
	return _anonIdentity;
}

void
EncryptionWPAEnterprise::setCertClient (const QString & certClient)
{
	kdDebug() << k_funcinfo << endl;
	_certClient = certClient;
}

QString
EncryptionWPAEnterprise::getCertClient (void) const
{
	kdDebug() << k_funcinfo << endl;
	return _certClient;
}

void
EncryptionWPAEnterprise::setCertCA (const QString & certCA)
{
	kdDebug() << k_funcinfo << endl;
	_certCA = certCA;
}

QString
EncryptionWPAEnterprise::getCertCA (void) const
{
	kdDebug() << k_funcinfo << endl;
	return _certCA;
}

void
EncryptionWPAEnterprise::setCertPrivate (const QString & certPrivate)
{
	kdDebug() << k_funcinfo << endl;
	_certPrivate = certPrivate;
}

QString
EncryptionWPAEnterprise::getCertPrivate (void) const
{
	kdDebug() << k_funcinfo << endl;
	return _certPrivate;
}

void
EncryptionWPAEnterprise::setProtocol (WPAProtocol protocol)
{
	kdDebug() << k_funcinfo << endl;
	_protocol = protocol;
}

WPAProtocol
EncryptionWPAEnterprise::getProtocol (void)
{
	kdDebug() << k_funcinfo << endl;
	return _protocol;
}

void
EncryptionWPAEnterprise::setVersion (WPAVersion version)
{
	kdDebug() << k_funcinfo <<  " " << version << endl;
	_version = version;
}

WPAVersion
EncryptionWPAEnterprise::getVersion (void)
{
	kdDebug() << k_funcinfo << endl;
	return _version;
}

void
EncryptionWPAEnterprise::setMethod (EAPMethod method)
{
	kdDebug() << k_funcinfo << " " << method <<  endl;
	_method = method;
}

EAPMethod
EncryptionWPAEnterprise::getMethod (void)
{
	kdDebug() << k_funcinfo << endl;
	return _method;
}

EncryptionWPAEnterprise::EncryptionWPAEnterprise ()
{
	kdDebug() << k_funcinfo << endl;
	/* no ciphers req. */

	setDefaults ();
}

EncryptionWPAEnterprise::~EncryptionWPAEnterprise ()
{
	kdDebug() << k_funcinfo << endl;

}
/*
	setMethod ((EAPMethod) method);
	setIdentity (identity);
	setAnonIdentity (anonIdentity);
	setCertPrivate (certPrivate);
	setCertClient (certClient);
	setCertCA (certCA);
	setVersion ((WPAVersion) version);

	EAP_PEAP = NM_EAP_METHOD_PEAP,
	EAP_TLS  = NM_EAP_METHOD_TLS,
	EAP_TTLS = NM_EAP_METHOD_TTLS
*/
void EncryptionWPAEnterprise::persist( KConfigBase * cfg ) const
{
	kdDebug() << k_funcinfo << endl;
	cfg->writeEntry( "Encryption", "WPA-EAP" );
	if ( EAP_PEAP == _method )
		cfg->writeEntry( "Method", "PEAP" );
	else if ( EAP_TLS == _method )
		cfg->writeEntry( "Method", "TLS" );
	else
		cfg->writeEntry( "Method", "TTLS" );
	
	cfg->writeEntry( "Identity", _identity );
	cfg->writeEntry( "AnonIdentity", _anonIdentity );
	cfg->writeEntry( "CertPrivate", _certPrivate );
	cfg->writeEntry( "CertClient", _certClient );
	cfg->writeEntry( "CertCA", _certCA );

	if ( WPA_TKIP == _protocol )
		cfg->writeEntry( "WPAProtocol", "TKIP" );
	else if (WPA_CCMP_AES == _protocol)
		cfg->writeEntry( "WPAProtocol", "CCMPAES" );
	
	if ( WPA1 == _version )
		cfg->writeEntry( "WPAVersion", "WPA1" );
	else
		cfg->writeEntry( "WPAVersion", "WPA2" );
}

void EncryptionWPAEnterprise::restore( const KConfigBase * cfg )
{
	kdDebug() << k_funcinfo << endl;

	QString wpaProtocol = cfg->readEntry( "WPAProtocol", "TKIP" );
	if ( "AUTO" == wpaProtocol )
		_protocol = WPA_AUTO;
	else if ( "TKIP" == wpaProtocol )
		_protocol = WPA_TKIP;
	else
		_protocol = WPA_CCMP_AES;
		
	QString wpaVersion = cfg->readEntry( "WPAVersion", "WPA1" );

	if ( "WPA1" == wpaVersion )
		_version = WPA1;
	else
		_version = WPA2;
		
	QString method = cfg->readEntry( "Method" );
	if ( "PEAP" == method )
		_method = EAP_PEAP;
	else if ( "TLS" == method )
		_method = EAP_TLS;
	else
		_method = EAP_TTLS;

	_identity = cfg->readEntry( "Identity" );
	_anonIdentity = cfg->readEntry( "AnonIdentity" );
	_certPrivate = cfg->readEntry( "CertPrivate" );
	_certClient = cfg->readEntry( "CertClient" );
	_certCA = cfg->readEntry( "CertCA" );
}
